# Functionality

In the [Introduction guide](#!/guide/introduction) you were introduced to the basic
functionality of the pivot grid. Below you will find more features available in this component.

## Calculation functions

The following calculation methods are supported out of the box: sum, avg, min, max, count,
groupSumPercentage, groupCountPercentage, variance, varianceP, stdDev and stdDevP.

You may ask yourself what is groupSumPercentage and groupCountPercentage?

* groupSumPercentage. If John sold items in USA and Germany you may want to see what percentage was
sold to USA from the total sales. So it actually calculates the percentage of the current item
sum out of the parent item sum.
* groupCountPercentage. Same as above except that count is used instead of sum.

If none of these standard functions cover your needs then you can define your own function.

    aggregate: [{
        // ...
        measure:    'field',
        aggregator: function(records, measure, matrix, rowGroupKey, colGroupKey) {
            // do your own algorithm
            return records.length;
        }
        // ...
    }]

If you want your grid to be stateless then we strongly suggest to override [Mz.aggregate.Aggregators](#!/api/Mz.aggregate.Aggregators)
to add your functions and use the function name in the aggregate dimension definition.

    Ext.define('overrides.aggregate.Aggregators', {
        override: 'Mz.aggregate.Aggregators',
        
        myOwnFn: function(records, measure, matrix, rowGroupKey, colGroupKey) {
            // do your own algorithm
            return records.length;
        }
    });
    // and define your aggregate dimension like this
    aggregate: [{
        // ...
        measure:    'field',
        aggregator: 'myOwnFn'
        // ...
    }]


## Sorting results

When you define a dimension item you can provide some configs that will change the way the
results are sorted.

* sortable: do you want to sort the results or not? Default is true.
* direction: you may specify if you want the results to be sorted ASC or DESC. Default is ASC.
* caseSensitiveSort: is this is case sensitive sort or not? Default is true.
* sorterFn: you can even provide your own sorting algorithm.
* sortIndex: sometimes you may want to sort your results using the value of another field 
on the record. Let's say you have one field "month-name" with values like Jan, Feb etc and
another field "month-value" with values like 1, 2 etc. It makes sense to have the results
from "month-name" (dataIndex) and sort by "month-value" (sortIndex).
    
## Multiple grand totals

By default the pivot grid calculates grand totals on rows and you may choose to show that or not.
If you want to add multiple grand totals on rows it is possible in two ways:

* You can use the event "pivotbuildtotals" fired on the pivot grid. The event provides a parameter
which is an array of grand total values including the default one. Each object in that array
should have a "title" and a "values" array that will be used to generate the new record.
* You can also extend one of the matrix classes and use the template method "onBuildTotals"
to do the same thing as above.

The new grand totals that you provide will be styled as the default grand total in the pivot grid.

    // ...
    listeners: {
        pivotbuildtotals: function(matrix, totals){
            var dataAvg = {},
                dataMax = {};
            
            Ext.Array.each(matrix.model, function(field){
                var result,
                    agg;
                
                if(field.col && field.agg){
                    agg = matrix.aggregate.getByKey(field.agg);
                    result = matrix.results.get(matrix.grandTotalKey, field.col);
                    if(result && agg){
                        dataAvg[field.name] = result.calculateByFn('totalavg', agg.dataIndex, Mz.aggregate.Aggregators.avg);
                        dataMax[field.name] = result.calculateByFn('totalmax', agg.dataIndex, Mz.aggregate.Aggregators.max);
                    }
                }
            });
            
            totals.push({
                title:  'Grand total (avg)',
                values: dataAvg
            },{
                title:  'Grand total (max)',
                values: dataMax
            });
        }
    }
    // ...

## Range grouping

Let's say that you have on one of the axis results for years and you want to group them
as 80', 90' and so on. You can achieve this in two ways:

* You can define a "grouperFn" on the dimension that will receive as argument the record
and should return the group value (i.e. 80', 90' etc).
* You can define a new field on the source model that has a "convert" function and does the same thing
as above.

Here is a usage example for grouperFn:

    leftAxis: [{
        // ...
        grouperFn: function(record){
            var dataIndex = this.dataIndex;
            if(record.get(dataIndex) >= 1980 && record.get(dataIndex) < 1990) return "80'";
            if(record.get(dataIndex) >= 1990 && record.get(dataIndex) < 2000) return "90'";
            return 'Rest';
        }
        // ...
    }]

## Server side calculations

The product comes with two types of calculations:

* Local calculations in the browser by using the [Local matrix](#!/api/Mz.aggregate.matrix.Local)
* Remote calculations on the server by using the [Remote matrix](#!/api/Mz.aggregate.matrix.Remote)

Both are explained in the [Client side API guide](#!/guide/api).

## Filtering

There are two types of filters available:

* Label filter: it allows you to filter the results by looking at the values generated for 
top or left axis results. i.e. begins with, does not begin with, ends with etc.
* Value filter: it allows you to filter the results by looking at the values calculated for each
top/left axis pair. i.e. equals to, greater than, top 10 items, top 10 percent, top 10 sum etc
    
The filter needs to be configured on a dimension item of the left or top axis.

    leftAxis: [{
        // ...
        filter: {
            mztype:      'value',
            type:       Mz.aggregate.filter.Value.TypeTop10,
            topOrder:   'top',
            topType:    'sum',
            value:      9500,
            dimensionId:'agg2'
        }
        // ...
    }]

To find out more have a look at the available filter classes (Mz.aggregate.filter.*).

## Buffered renderer

When the calculations are ready inside the pivot grid there could be thousand of records available
in the pivot store. This may be a problem for grid rendering but fortunately the Ext JS
framework has a BufferedRenderer plugin available since version 4.2.0. In version 5.0.x this
plugin is used by the grid panel by default.

This plugin allows users to scroll through thousand of records without the performance penalties
of rendering all the records on screen at once.

This is how you can use it:

    var pivot = Ext.create('Mz.pivot.Grid', {
        // ...
        plugins: {
            ptype: 'bufferedrenderer',
            trailingBufferZone: 20,  // Keep 20 rows rendered in the table behind scroll
            leadingBufferZone: 50   // Keep 50 rows rendered in the table ahead of scroll
        },
        // ...
    });

